@@ -19,5 +19,6 @@
 // DEALINGS IN THE SOFTWARE.
 
 fn main() {
-    prost_build::compile_protos(&["src/message.proto"], &["src"]).unwrap();
+    prost_build::compile_protos(&["src/v1/message.proto"], &["src/v1"]).unwrap();
+    prost_build::compile_protos(&["src/v2/message.proto"], &["src/v2"]).unwrap();
 }
