@@ -31,7 +31,7 @@ use libp2p_core::multiaddr::Multiaddr;
 use libp2p_core::PeerId;
 use libp2p_swarm::{
     dial_opts::{self, DialOpts},
-    DialError, IntoProtocolsHandler, NetworkBehaviour, NetworkBehaviourAction, NotifyHandler,
+    DialError, IntoConnectionHandler, NetworkBehaviour, NetworkBehaviourAction, NotifyHandler,
     PollParameters,
 };
 use std::collections::{hash_map::Entry, HashMap, HashSet, VecDeque};
@@ -102,7 +102,7 @@ enum IncomingRelayReq {
     },
 }
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct RelayConfig {
     /// How long to keep connections alive when they're idle.
     ///
@@ -154,10 +154,10 @@ impl Relay {
 }
 
 impl NetworkBehaviour for Relay {
-    type ProtocolsHandler = RelayHandlerProto;
+    type ConnectionHandler = RelayHandlerProto;
     type OutEvent = ();
 
-    fn new_handler(&mut self) -> Self::ProtocolsHandler {
+    fn new_handler(&mut self) -> Self::ConnectionHandler {
         RelayHandlerProto {
             config: RelayHandlerConfig {
                 connection_idle_timeout: self.config.connection_idle_timeout,
@@ -205,6 +205,7 @@ impl NetworkBehaviour for Relay {
         connection_id: &ConnectionId,
         _: &ConnectedPoint,
         _: Option<&Vec<Multiaddr>>,
+        other_established: usize,
     ) {
         let is_first = self
             .connected_peers
@@ -238,69 +239,61 @@ impl NetworkBehaviour for Relay {
                 },
             );
         }
-    }
 
-    fn inject_connected(&mut self, peer_id: &PeerId) {
-        assert!(
-            self.connected_peers
-                .get(peer_id)
-                .map(|cs| !cs.is_empty())
-                .unwrap_or(false),
-            "Expect to be connected to peer with at least one connection."
-        );
+        if other_established == 0 {
+            if let Some(reqs) = self.outgoing_relay_reqs.dialing.remove(peer) {
+                for req in reqs {
+                    let OutgoingDialingRelayReq {
+                        request_id,
+                        src_peer_id,
+                        relay_addr: _,
+                        dst_addr,
+                        dst_peer_id,
+                        send_back,
+                    } = req;
+                    self.outbox_to_swarm
+                        .push_back(NetworkBehaviourAction::NotifyHandler {
+                            peer_id: *peer,
+                            handler: NotifyHandler::Any,
+                            event: RelayHandlerIn::OutgoingRelayReq {
+                                src_peer_id,
+                                request_id,
+                                dst_peer_id,
+                                dst_addr: dst_addr.clone(),
+                            },
+                        });
 
-        if let Some(reqs) = self.outgoing_relay_reqs.dialing.remove(peer_id) {
-            for req in reqs {
-                let OutgoingDialingRelayReq {
-                    request_id,
-                    src_peer_id,
-                    relay_addr: _,
-                    dst_addr,
-                    dst_peer_id,
-                    send_back,
-                } = req;
-                self.outbox_to_swarm
-                    .push_back(NetworkBehaviourAction::NotifyHandler {
-                        peer_id: *peer_id,
-                        handler: NotifyHandler::Any,
-                        event: RelayHandlerIn::OutgoingRelayReq {
-                            src_peer_id,
-                            request_id,
-                            dst_peer_id,
-                            dst_addr: dst_addr.clone(),
-                        },
-                    });
-
-                self.outgoing_relay_reqs
-                    .upgrading
-                    .insert(request_id, OutgoingUpgradingRelayReq { send_back });
+                    self.outgoing_relay_reqs
+                        .upgrading
+                        .insert(request_id, OutgoingUpgradingRelayReq { send_back });
+                }
             }
-        }
 
-        // Ask the newly-opened connection to be used as destination if relevant.
-        if let Some(reqs) = self.incoming_relay_reqs.remove(peer_id) {
-            for req in reqs {
-                let IncomingRelayReq::DialingDst {
-                    src_peer_id,
-                    src_addr,
-                    src_connection_id,
-                    request_id,
-                    incoming_relay_req,
-                } = req;
-                let event = RelayHandlerIn::OutgoingDstReq {
-                    src_peer_id,
-                    src_addr,
-                    src_connection_id,
-                    request_id,
-                    incoming_relay_req,
-                };
+            // Ask the newly-opened connection to be used as destination if relevant.
+            if let Some(reqs) = self.incoming_relay_reqs.remove(peer) {
+                for req in reqs {
+                    let IncomingRelayReq::DialingDst {
+                        src_peer_id,
+                        src_addr,
+                        src_connection_id,
+                        request_id,
+                        incoming_relay_req,
+                    } = req;
+                    let event = RelayHandlerIn::OutgoingDstReq {
+                        src_peer_id,
+                        src_addr,
+                        src_connection_id,
+                        request_id,
+                        incoming_relay_req,
+                    };
 
-                self.outbox_to_swarm
-                    .push_back(NetworkBehaviourAction::NotifyHandler {
-                        peer_id: *peer_id,
-                        handler: NotifyHandler::Any,
-                        event,
-                    });
+                    self.outbox_to_swarm
+                        .push_back(NetworkBehaviourAction::NotifyHandler {
+                            peer_id: *peer,
+                            handler: NotifyHandler::Any,
+                            event,
+                        });
+                }
             }
         }
     }
@@ -308,7 +301,7 @@ impl NetworkBehaviour for Relay {
     fn inject_dial_failure(
         &mut self,
         peer_id: Option<PeerId>,
-        _: Self::ProtocolsHandler,
+        _: Self::ConnectionHandler,
         error: &DialError,
     ) {
         if let DialError::DialPeerConditionFalse(
@@ -359,7 +352,8 @@ impl NetworkBehaviour for Relay {
         peer: &PeerId,
         connection: &ConnectionId,
         _: &ConnectedPoint,
-        _: <Self::ProtocolsHandler as IntoProtocolsHandler>::Handler,
+        _: <Self::ConnectionHandler as IntoConnectionHandler>::Handler,
+        remaining_established: usize,
     ) {
         // Remove connection from the set of connections for the given peer. In case the set is
         // empty it will be removed in `inject_disconnected`.
@@ -414,6 +408,28 @@ impl NetworkBehaviour for Relay {
                 }
             }
         }
+
+        if remaining_established == 0 {
+            self.connected_peers.remove(peer);
+
+            if let Some(reqs) = self.incoming_relay_reqs.remove(peer) {
+                for req in reqs {
+                    let IncomingRelayReq::DialingDst {
+                        src_peer_id,
+                        incoming_relay_req,
+                        ..
+                    } = req;
+                    self.outbox_to_swarm
+                        .push_back(NetworkBehaviourAction::NotifyHandler {
+                            peer_id: src_peer_id,
+                            handler: NotifyHandler::Any,
+                            event: RelayHandlerIn::DenyIncomingRelayReq(
+                                incoming_relay_req.deny(circuit_relay::Status::HopCantDialDst),
+                            ),
+                        })
+                }
+            }
+        }
     }
 
     fn inject_listener_error(&mut self, _id: ListenerId, _err: &(dyn std::error::Error + 'static)) {
@@ -421,28 +437,6 @@ impl NetworkBehaviour for Relay {
 
     fn inject_listener_closed(&mut self, _id: ListenerId, _reason: Result<(), &std::io::Error>) {}
 
-    fn inject_disconnected(&mut self, id: &PeerId) {
-        self.connected_peers.remove(id);
-
-        if let Some(reqs) = self.incoming_relay_reqs.remove(id) {
-            for req in reqs {
-                let IncomingRelayReq::DialingDst {
-                    src_peer_id,
-                    incoming_relay_req,
-                    ..
-                } = req;
-                self.outbox_to_swarm
-                    .push_back(NetworkBehaviourAction::NotifyHandler {
-                        peer_id: src_peer_id,
-                        handler: NotifyHandler::Any,
-                        event: RelayHandlerIn::DenyIncomingRelayReq(
-                            incoming_relay_req.deny(circuit_relay::Status::HopCantDialDst),
-                        ),
-                    })
-            }
-        }
-    }
-
     fn inject_event(
         &mut self,
         event_source: PeerId,
@@ -576,7 +570,7 @@ impl NetworkBehaviour for Relay {
         &mut self,
         cx: &mut Context<'_>,
         poll_parameters: &mut impl PollParameters,
-    ) -> Poll<NetworkBehaviourAction<Self::OutEvent, Self::ProtocolsHandler>> {
+    ) -> Poll<NetworkBehaviourAction<Self::OutEvent, Self::ConnectionHandler>> {
         if !self.outbox_to_listeners.is_empty() {
             let relay_peer_id = self.outbox_to_listeners[0].0;
 
