@@ -30,12 +30,12 @@ use instant::Instant;
 use libp2p_core::connection::{ConnectedPoint, ConnectionId};
 use libp2p_core::multiaddr::Protocol;
 use libp2p_core::PeerId;
-use libp2p_swarm::protocols_handler::DummyProtocolsHandler;
+use libp2p_swarm::handler::DummyConnectionHandler;
 use libp2p_swarm::{
-    NetworkBehaviour, NetworkBehaviourAction, NotifyHandler, PollParameters,
-    ProtocolsHandlerUpgrErr,
+    ConnectionHandlerUpgrErr, NetworkBehaviour, NetworkBehaviourAction, NotifyHandler,
+    PollParameters,
 };
-use std::collections::{HashMap, HashSet, VecDeque};
+use std::collections::{hash_map, HashMap, HashSet, VecDeque};
 use std::num::NonZeroU32;
 use std::ops::Add;
 use std::task::{Context, Poll};
@@ -61,6 +61,28 @@ pub struct Config {
     pub circuit_src_rate_limiters: Vec<Box<dyn rate_limiter::RateLimiter>>,
 }
 
+impl std::fmt::Debug for Config {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        f.debug_struct("Config")
+            .field("max_reservations", &self.max_reservations)
+            .field("max_reservations_per_peer", &self.max_reservations_per_peer)
+            .field("reservation_duration", &self.reservation_duration)
+            .field(
+                "reservation_rate_limiters",
+                &format!("[{} rate limiters]", self.reservation_rate_limiters.len()),
+            )
+            .field("max_circuits", &self.max_circuits)
+            .field("max_circuits_per_peer", &self.max_circuits_per_peer)
+            .field("max_circuit_duration", &self.max_circuit_duration)
+            .field("max_circuit_bytes", &self.max_circuit_bytes)
+            .field(
+                "circuit_src_rate_limiters",
+                &format!("[{} rate limiters]", self.circuit_src_rate_limiters.len()),
+            )
+            .finish()
+    }
+}
+
 impl Default for Config {
     fn default() -> Self {
         let reservation_rate_limiters = vec![
@@ -129,7 +151,7 @@ pub enum Event {
     ReservationTimedOut { src_peer_id: PeerId },
     CircuitReqReceiveFailed {
         src_peer_id: PeerId,
-        error: ProtocolsHandlerUpgrErr<void::Void>,
+        error: ConnectionHandlerUpgrErr<void::Void>,
     },
     /// An inbound circuit request has been denied.
     CircuitReqDenied {
@@ -151,7 +173,7 @@ pub enum Event {
     CircuitReqOutboundConnectFailed {
         src_peer_id: PeerId,
         dst_peer_id: PeerId,
-        error: ProtocolsHandlerUpgrErr<outbound_stop::CircuitFailedReason>,
+        error: ConnectionHandlerUpgrErr<outbound_stop::CircuitFailedReason>,
     },
     /// Accepting an inbound circuit request failed.
     CircuitReqAcceptFailed {
@@ -194,10 +216,10 @@ impl Relay {
 }
 
 impl NetworkBehaviour for Relay {
-    type ProtocolsHandler = handler::Prototype;
+    type ConnectionHandler = handler::Prototype;
     type OutEvent = Event;
 
-    fn new_handler(&mut self) -> Self::ProtocolsHandler {
+    fn new_handler(&mut self) -> Self::ConnectionHandler {
         handler::Prototype {
             config: handler::Config {
                 reservation_duration: self.config.reservation_duration,
@@ -212,10 +234,14 @@ impl NetworkBehaviour for Relay {
         peer: &PeerId,
         connection: &ConnectionId,
         _: &ConnectedPoint,
-        _handler: Either<handler::Handler, DummyProtocolsHandler>,
+        _handler: Either<handler::Handler, DummyConnectionHandler>,
+        _remaining_established: usize,
     ) {
-        if let Some(connections) = self.reservations.get_mut(peer) {
-            connections.remove(&connection);
+        if let hash_map::Entry::Occupied(mut peer) = self.reservations.entry(*peer) {
+            peer.get_mut().remove(&connection);
+            if peer.get().is_empty() {
+                peer.remove();
+            }
         }
 
         for circuit in self
@@ -257,7 +283,7 @@ impl NetworkBehaviour for Relay {
 
                 assert!(
                     !endpoint.is_relayed(),
-                    "`DummyProtocolsHandler` handles relayed connections. It \
+                    "`DummyConnectionHandler` handles relayed connections. It \
                      denies all inbound substreams."
                 );
 
@@ -353,9 +379,21 @@ impl NetworkBehaviour for Relay {
                 );
             }
             handler::Event::ReservationTimedOut {} => {
-                self.reservations
-                    .get_mut(&event_source)
-                    .map(|cs| cs.remove(&connection));
+                match self.reservations.entry(event_source) {
+                    hash_map::Entry::Occupied(mut peer) => {
+                        peer.get_mut().remove(&connection);
+                        if peer.get().is_empty() {
+                            peer.remove();
+                        }
+                    }
+                    hash_map::Entry::Vacant(_) => {
+                        unreachable!(
+                            "Expect to track timed out reservation with peer {:?} on connection {:?}",
+                            event_source,
+                            connection,
+                        );
+                    }
+                }
 
                 self.queued_actions.push_back(
                     NetworkBehaviourAction::GenerateEvent(Event::ReservationTimedOut {
@@ -372,7 +410,7 @@ impl NetworkBehaviour for Relay {
 
                 assert!(
                     !endpoint.is_relayed(),
-                    "`DummyProtocolsHandler` handles relayed connections. It \
+                    "`DummyConnectionHandler` handles relayed connections. It \
                      denies all inbound substreams."
                 );
 
@@ -584,7 +622,7 @@ impl NetworkBehaviour for Relay {
         &mut self,
         _cx: &mut Context<'_>,
         poll_parameters: &mut impl PollParameters,
-    ) -> Poll<NetworkBehaviourAction<Self::OutEvent, Self::ProtocolsHandler>> {
+    ) -> Poll<NetworkBehaviourAction<Self::OutEvent, Self::ConnectionHandler>> {
         if let Some(action) = self.queued_actions.pop_front() {
             return Poll::Ready(action.build(poll_parameters));
         }
